/******************************************************************************
  * @file    PT32X003_pwr.c
  * @author  ӦÿŶ
  * @version V1.0.0
  * @date    2022/5/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the Power Controller (PWR) peripheral:
  *           + Backup Domain Access
  *           + WakeUp pins configuration
  *           + Low Power modes configuration
  *           + Flags management
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/


/* Includes ------------------------------------------------------------------*/
#include "PT32X003_pwr.h"


/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/





/**
  * @brief  Enters Sleep mode.
  * @param  PWR_SLEEPEntry: specifies if SLEEP mode in entered with WFI or WFE instruction.
  *          This parameter can be one of the following values:
  *             @arg PWR_SLEEPEntry_WFI: enter SLEEP mode with WFI instruction
  *             @arg PWR_SLEEPEntry_WFE: enter SLEEP mode with WFE instruction
  * @retval None
  */
void PWR_EnterSleepMode(u8 PWR_SLEEPEntry)
{
	/* Check the parameters */
	assert_param(IS_PWR_SLEEP_ENTRY(PWR_SLEEPEntry));
	/* Clear SLEEPDEEP bit of Cortex-M0 System Control Register */
	SCB->SCR &= (u32)~((u32)SCB_SCR_SLEEPDEEP_Msk);
	/* Select SLEEP mode entry -------------------------------------------------*/
	if(PWR_SLEEPEntry == PWR_SLEEPEntry_WFI)
	{
		/* Request Wait For Interrupt */
		__WFI();
	}
	else
	{
		/* Request Wait For Event */
		__SEV();
		__WFE();
		__WFE();
	}
}



/**
  * @brief  Enters STANDBY mode.
  * @param  PWR_DeepStandbyEntry: specifies if SLEEP mode in entered with WFI or WFE instruction.
  *          This parameter can be one of the following values:
  *             @arg PWR_STANDBYEntry_WFI: enter DEEPSLEEP mode with WFI instruction
  *             @arg PWR_STANDBYEntry_WFE: enter DEEPSLEEP mode with WFE instruction
  * @retval None
  */
void PWR_EnterSTANDBYMode(u8 PWR_DeepStandbyEntry)
{
	/* Check the parameters */
	assert_param(IS_PWR_STANDBY_ENTRY(PWR_DeepStandbyEntry));
	/* Set SLEEPDEEP bit of Cortex-M0 System Control Register */
	SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;
	/* Request Wait For Interrupt */
	/* Select SLEEP mode entry -------------------------------------------------*/
	if(PWR_DeepStandbyEntry == PWR_STANDBYEntry_WFI)
	{
		/* Request Wait For Interrupt */
		__WFI();
	}
	else
	{
		/* Request Wait For Event */
		__WFE();
		__WFE();
	}
}

/**
  * @}
  */

