  /******************************************************************************
  * @file    PT32X003_rcc.c
  * @author  ?????????
  * @version V1.0.0
  * @date    2022/5/27
  * @brief    This file provides firmware functions to manage the following
  *          functionalities of the SYS peripheral:
  *           + Initialization and Configuration
  *           + Interrupts and flags management
  *            
  ******************************************************************************
  * @attention
  *
  *
  *****************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "PT32X003_rcc.h"

#define  HSI   48000000UL
#define  LSI   128000UL


/** @defgroup RCC
  * @brief RCC driver modules
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/**
  * @brief  Gets the SYS_CLK value.
  * @param  None
  * @retval SYS_CLK value
  */
u32 RCC_GetSYSCLKFreq(void)
{
	return HSI;
}

/**
  * @brief  Gets the HCLK_CLK value.
  * @param  None
  * @retval HCLK_CLK value
  */
u32 RCC_GetHCLKFreq(void)
{
    u32 HCLK_Freq = 0;
    u32 HCLK_DIV = 0;
    
    HCLK_DIV = ((RCC->CFGR & RCC_CFGR_HPRE) >> 4) + 1;
    HCLK_Freq = HSI / HCLK_DIV;
    return HCLK_Freq;
}

/**
  * @brief  Gets the PCLK_CLK value.
  * @param  None
  * @retval PCLK_CLK value
  */
u32 RCC_GetPCLKFreq(void)
{
    u32 PCLK_Freq = 0;
    u32 PCLK_DIV = 0;
    
    u32 HCLK_Freq = RCC_GetHCLKFreq();
    PCLK_DIV = ((RCC->CFGR & RCC_CFGR_PPRE) >> 24) + 1;
    PCLK_Freq = HCLK_Freq / PCLK_DIV;
    
    return PCLK_Freq;
      
}


/**
  * @brief  Check whether the specific global reset flag is set or not.
 *  @param RSTCU_RSTF: specify the reset flag.
 *        This parameter can be:
 *        @arg RCC_RSR_SFR
 *        @arg RCC_RSR_IWDGR
 *        @arg RCC_RSR_POR
 *        @arg RCC_RSR_PINR
 *  @retval SET or RESET
  */
FlagStatus RCC_GetResetFlagStatus(u32 RSTCU_RSTF)
{
	u32 tmp;
	/* Check the parameters */
	assert_param(IS_RSTCU_FLAG(RSTCU_RSTF));
	tmp = (RCC->RSR & (RSTCU_RSTF));
	if (tmp != RESET)
	{
		return SET;
	}
	else
	{
		return RESET;
	}
}

/**
 * @brief  Clear the specific global reset flag.
 * @param RSTCU_RSTF: specify the reset flag.
 *        This parameter can be:
 *        @arg RCC_RSR_SFR
 *        @arg RCC_RSR_IWDGR
 *        @arg RCC_RSR_POR
 *        @arg RCC_RSR_PINR
 * @retval None
  */
void RCC_ClearResetFlag(u32 RSTCU_RSTF)
{
	/* Check the parameters */
	assert_param(IS_RSTCU_FLAG(RSTCU_RSTF));
	RCC->RSR = RSTCU_RSTF; 
}


/**
  * @brief  Enables or disables the specified Reset Source.
  * @param  RstSource: specifies the Reset source to be enabled or disabled.
  *          This parameter can be one of the following values:
 *        @arg RCC_RSR_SFR
 *        @arg RCC_RSR_IWDGR
 *        @arg RCC_RSR_POR
 *        @arg RCC_RSR_PINR
  * @param  NewState: new state of the specified RSTEN SOURCE
  *          This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void RCC_RestConfig(u32 RstSource, FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_FUNCTIONAL_STATE(NewState));
	assert_param(IS_RSTEN_SOURCE(RstSource));
	if (NewState != DISABLE)
	{
		RCC->RCR |= RstSource;
	}
	else
	{
		RCC->RCR &= (u16)~RstSource;
	}
}


/**
  * @}
  */

